//
//  BubbleSprite.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 25.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "BubbleSprite.h"

@implementation BubbleSprite
{
    SKSpriteNode* _cornerLU;
    SKSpriteNode* _cornerLD;
    SKSpriteNode* _cornerRU;
    SKSpriteNode* _cornerRD;
    SKSpriteNode* _borderL;
    SKSpriteNode* _borderR;
    SKSpriteNode* _borderU;
    SKSpriteNode* _borderD;
    SKSpriteNode* _center;
    SKSpriteNode* _pointer;
}

- (instancetype)initWithContentList:(NSArray*)contentList
{
    CGRect sumFrame = {0, 0, 0, 0};
    for (SKSpriteNode* content in contentList) {
        if (CGSizeEqualToSize(sumFrame.size, CGSizeZero)) {
            sumFrame = content.frame;
        }
        else {
            sumFrame = CGRectUnion(sumFrame, content.frame);
        }
    }
    
    return [self initWithFrame:sumFrame];
}

- (instancetype)initWithContent:(SKSpriteNode*)content
{
    return [self initWithFrame:content.frame];
}

- (instancetype)initWithFrame:(CGRect)frame
{
    frame = CGRectInset(frame, -40, -40);
    self = [self initWithSize:frame.size];
    
    if (self != nil) {
        CGPoint frameCenter = CGPointMake(CGRectGetMidX(frame), CGRectGetMidY(frame));
        self.position = CGPointMake([self insertToNet:frameCenter.x], [self insertToNet:frameCenter.y]);
    }
    
    return self;
}

- (instancetype)initWithSize:(CGSize)size
{
    self = [super init];
    
    if (self != nil) {
        size = CGSizeMake([self insertToNet:size.width], [self insertToNet:size.height]);
        
        _cornerLU = [self loadSprite:@"corner_ul"];
        _cornerLD = [self loadSprite:@"corner_dl"];
        _cornerRU = [self loadSprite:@"corner_ur"];
        _cornerRD = [self loadSprite:@"corner_dr"];
        _borderL = [self loadSprite:@"border_l"];
        _borderR = [self loadSprite:@"border_r"];
        _borderU = [self loadSprite:@"border_u"];
        _borderD = [self loadSprite:@"border_d"];
        _center = [self loadSprite:@"center"];
        _pointer = [self loadSprite:@"bubble_pointer"];
        
        _cornerLU.position = CGPointMake(0.5 * (_cornerLU.size.width - size.width), 0.5 * (size.height - _cornerLU.size.height));
        _cornerLD.position = CGPointMake(0.5 * (_cornerLU.size.width - size.width), 0.5 * (_cornerLU.size.height - size.height));
        _cornerRU.position = CGPointMake(0.5 * (size.width - _cornerLU.size.width), 0.5 * (size.height - _cornerLU.size.height));
        _cornerRD.position = CGPointMake(0.5 * (size.width - _cornerLU.size.width), 0.5 * (_cornerLU.size.height - size.height));
        
        _borderL.position = CGPointMake(0.5 * (_borderL.size.width - size.width), 0.5 * (_cornerLD.size.height - _cornerLU.size.height));
        _borderL.yScale = (size.height - _cornerLD.size.height - _cornerLU.size.height) / _borderL.size.height;
        _borderR.position = CGPointMake(0.5 * (size.width - _borderR.size.width), 0.5 * (_cornerRD.size.height - _cornerRU.size.height));
        _borderR.yScale = (size.height - _cornerRD.size.height - _cornerRU.size.height) / _borderR.size.height;
        _borderU.position = CGPointMake(0.5 * (_cornerLU.size.width - _cornerRU.size.width), 0.5 * (size.height - _borderU.size.height));
        _borderU.xScale = (size.width - _cornerLU.size.width - _cornerLU.size.width) / _borderU.size.width;
        _borderD.position = CGPointMake(0.5 * (_cornerLD.size.width - _cornerRD.size.width), 0.5 * (_borderD.size.height - size.height));
        _borderD.xScale = (size.width - _cornerLD.size.width - _cornerLD.size.width) / _borderD.size.width;
        
        _center.position = CGPointMake(0.5 * (_borderL.size.width - _borderR.size.width), 0.5 * (_borderD.size.height - _borderU.size.height));
        _center.size = CGSizeMake(size.width - _borderL.size.width - _borderR.size.width, size.height - _borderD.size.height - _borderU.size.height);
        
        _pointer.position = CGPointMake(-0.3 * size.width, 4 - 0.5 * (size.height + _pointer.size.height));
    }
    
    return self;
}

- (SKSpriteNode*)loadSprite:(NSString*)name
{
    SKSpriteNode* sprite = [SKSpriteNode spriteNodeWithImageNamed:name];
    [self addChild:sprite];
    
    return sprite;
}

- (CGFloat)insertToNet:(CGFloat)coordinate
{
    return 8 * roundf(0.125 * coordinate);
}

@end
