//
//  Character.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "Character.h"
#import "NameManager.h"

static NSArray *spriteList = nil;

@implementation Character
{
    Money _goodActionCost;
    Money _badActionCost;
    SKTexture* _standTexture;
    SKTexture* _winkTexture;
}

- (instancetype)init
{
    return [self initWithType:arc4random() % 4];
}

- (instancetype) initWithType: (CharacterType) type
{
    if (spriteList == nil) {
        spriteList = @[@"knight", @"priest", @"mage", @"jester"];
    }
    
    self = [super init];
    
    if (self != nil) {
        _type = type;
        _name = [[NameManager sharedManager] characterNameForType:_type];
        
        _salary = 750 + 10 * (arc4random() % 75);
        _goodActionCost = 300 + 10 * (arc4random() % 35);
        _badActionCost = 300 + 10 * (arc4random() % 30);
        
        NSString *filename = spriteList[_type];
        _standTexture = [SKTexture textureWithImageNamed:[NSString stringWithFormat:@"%@_stand", filename]];
        _winkTexture = [SKTexture textureWithImageNamed:[NSString stringWithFormat:@"%@_wink", filename]];
        
        _sprite = [SKSpriteNode spriteNodeWithTexture:_standTexture];
    }
    
    return self;
}

- (void)getNewByname:(NSString *)key
{
    _byname = [[NameManager sharedManager] bynameByStatisticKey:key andCharacterType:self.type];
}

- (void)upgradeLevel
{
    _salary += 10 * (5 + arc4random() % 15);
    _goodActionCost += 10 * (2 + arc4random() % 5);
    _badActionCost += 10 * (2 + arc4random() % 5);
}

- (Money)goodActionCost
{
    if (self.specialization > 0) {
        return _goodActionCost * (1 + 0.1 * self.specialization);
    }
    
    return _goodActionCost;
}

- (Money)badActionCost
{
    if (self.specialization < 0) {
        return _badActionCost * (1 - 0.1 * self.specialization);
    }
    
    return _badActionCost;
}

- (void)wink
{
    [_sprite removeAllActions];

    SKAction* delay1Action = [SKAction waitForDuration:0.008 * (arc4random() % 100) + 0.5];
    SKAction* winkAction = [SKAction setTexture:_winkTexture];
    SKAction* delay2Action = [SKAction waitForDuration:0.3];
    SKAction* idleAction = [SKAction setTexture:_standTexture];
    SKAction* sequence = [SKAction sequence:@[delay1Action, winkAction, delay2Action, idleAction]];
    
    [_sprite runAction:sequence completion:^{
        [self wink];
    }];
}

@end
