//
//  CharacterManager.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "CharacterManager.h"
#import "CharacterMenu.h"
#import "King.h"

static const int CHARACTER_AMOUNT = 4;

@implementation CharacterManager
{
    NSMutableArray* _characterList;
}

+ (instancetype)sharedManager
{
    static CharacterManager* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        instance = [CharacterManager new];
    });
    
    return instance;
}

- (instancetype)init
{
    self = [super init];
    
    if (self != nil) {
        [self reset];
    }
    
    return self;
}

- (NSArray *) characterList
{
    return _characterList;
}

- (Money)salaryForAllCharacters
{
    Money total = 0;
    for (Character* character in _characterList) {
        total += character.salary;
    }
    
    return total;
}

- (void)returnCharacters
{
    for (Character* character in _characterList) {
        character.isBusy = NO;
    }
}

- (void)hireCharacter:(CharacterType)type
{
    Character* newCharacter = [[Character alloc] initWithType:type];
    [_characterList removeObject:newCharacter];
    
    [[CharacterMenu sharedMenu] updateMenu];
}

- (void)killCharacter:(Character *)character
{
    [_characterList removeObject:character];
}

- (void)fireCharacter:(Character *)character
{
    [_characterList removeObject:character];
}

- (void)reset
{
    _characterList = [NSMutableArray new];
    
    for (NSUInteger characterIndex = 0; characterIndex < CHARACTER_AMOUNT; characterIndex++) {
        [_characterList addObject:[[Character alloc] initWithType:(CharacterType) characterIndex]];
    }
    
    _king = [King new];
}

@end
