//
//  CharacterMenu.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "CharacterMenu.h"
#import "CharacterManager.h"
#import "CharacterNode.h"
#import "ThroneScene.h"
#import "DSMultilineLabelNode.h"

@implementation CharacterMenu
{
    SKSpriteNode* _backButton;
    SKSpriteNode* _selectionSprite;
    
    NSMutableArray* _characterList;
    NSUInteger _selection;
}

+ (instancetype)sharedMenu
{
    static CharacterMenu* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        instance = [CharacterMenu new];
        instance.userInteractionEnabled = YES;
    });
    
    return instance;
}

- (instancetype)init
{
    self = [super init];
    
    if (self != nil) {
        _selection = NSNotFound;
    }
    
    return self;
}

- (void)initMenuInterface
{
    SKSpriteNode* menuBackground = [SKSpriteNode spriteNodeWithColor:[UIColor blackColor] size:self.scene.size];
    
    [self addChild:menuBackground];
    menuBackground.zPosition = 1;
    _backButton = [SKSpriteNode new];
    _backButton.position = CGPointMake(-0.47 * self.scene.size.width, 0.47 * self.scene.size.height);
    _backButton.zPosition = 20;
    [self addChild:_backButton];
    
    _selectionSprite = [SKSpriteNode spriteNodeWithImageNamed:@"selector"];
    _selectionSprite.zPosition = 25;
    
    DSMultilineLabelNode *titleNode = [DSMultilineLabelNode new];
    //titleNode.text = @"Call a patrial!";
    titleNode.text = @"Призвать вассала!";
    titleNode.position = CGPointMake(0, 0.3 * self.scene.size.height);
    titleNode.fontColor = [UIColor whiteColor];
    titleNode.zPosition = 2;
    titleNode.fontSize = 35;
    [self addChild:titleNode];
}

- (void)updateMenu
{
    if (_backButton == nil) {
        [self initMenuInterface];
    }
    
    if (_characterList == nil) {
        _characterList = [NSMutableArray new];
    }
    
    NSArray* characters = [CharacterManager sharedManager].characterList;
    CGFloat offset = 0.7 / (characters.count - 1);
    for (NSUInteger characterIndex = 0; characterIndex < characters.count || characterIndex < _characterList.count; characterIndex++) {
        
        if (characterIndex >= characters.count) {
            CharacterNode* characterNode = _characterList[characterIndex];
            [characterNode removeFromParent];
            
            [_characterList removeObjectAtIndex:characterIndex];
            characterIndex--;
            
            continue;
        }
        
        Character* character = characters[characterIndex];
        
        CharacterNode* characterNode = nil;
        if (characterIndex < _characterList.count) {
            characterNode = _characterList[characterIndex];
        }
        else {
            characterNode = [CharacterNode new];
            
            characterNode.goldDisplay = GoldDisplayActionCost;
            characterNode.zPosition = 20;
            
            [_characterList addObject:characterNode];
            [self addChild:characterNode];
        }
        
        characterNode.character = character;
        characterNode.position = CGPointMake((-0.35 + offset * characterIndex) * self.scene.size.width, -0.05 * self.scene.size.height);
        
        [character wink];
    }
    
    _selection = NSNotFound;
    [_selectionSprite removeFromParent];
}

- (Character *)selectedCharacter
{
    NSArray* characters = [CharacterManager sharedManager].characterList;
    if (characters.count > _selection) {
        return characters[_selection];
    }
    else {
        return nil;
    }
}

#pragma mark- Touch methods

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    UITouch* touch = [touches anyObject];
    CGPoint touchPoint = [touch locationInNode:self];
    
    if (touchPoint.x < -0.4 * self.scene.size.width && touchPoint.y > 0.4 * self.scene.size.height) {
        _selection = NSNotFound;
        
        [self removeFromParent];
        [[ThroneScene sharedScene] returnToThrone];
        
        return;
    }
    
    for (NSUInteger characterIndex = 0; characterIndex < _characterList.count; characterIndex++) {
        Character* character = [CharacterManager sharedManager].characterList[characterIndex];
        if (character.isBusy) {
            continue;
        }
        
        SKNode* characterNode = _characterList[characterIndex];
        CGRect characterFrame = characterNode.calculateAccumulatedFrame;
        if (CGRectContainsPoint(characterFrame, touchPoint)) {
            if (_selection == characterIndex) {
                [self removeFromParent];
                [[ThroneScene sharedScene] returnToThrone];
            }
            else {
                _selection = characterIndex;
                
                if (_selectionSprite.parent == nil) {
                    [self addChild:_selectionSprite];
                }
                
                _selectionSprite.position = characterNode.position;
                _selectionSprite.size = characterFrame.size;
            }
            
            break;
        }
    }
}

@end
