//
//  CharacterNode.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "CharacterNode.h"
#import "DSMultilineLabelNode.h"
#import "MessengerSprite.h"

static CGSize const CharacterNodeSize = {160, 320};

@implementation CharacterNode
{
    DSMultilineLabelNode* _nameLabel;
    SKSpriteNode* _imageNode;
    DSMultilineLabelNode* _goodLabel;
    DSMultilineLabelNode* _badLabel;
    
    SKSpriteNode* _grayField;
}

- (instancetype)init
{
    self = [super init];
    
    if (self != nil) {
        _nameLabel = [DSMultilineLabelNode new];
        _nameLabel.fontColor = [UIColor whiteColor];
        _nameLabel.zPosition = 10;
        _nameLabel.paragraphWidth = CharacterNodeSize.width;
        
        [self addChild:_nameLabel];
        
        _goodLabel = [DSMultilineLabelNode new];
        _goodLabel.fontColor = [UIColor greenColor];
        _goodLabel.fontSize = 16;
        _goodLabel.zPosition = 10;
        _goodLabel.paragraphWidth = CharacterNodeSize.width;
        
        [self addChild:_goodLabel];
        
        _badLabel = [DSMultilineLabelNode new];
        _badLabel.fontColor = [UIColor redColor];
        _badLabel.fontSize = 16;
        _badLabel.zPosition = 10;
        _badLabel.paragraphWidth = CharacterNodeSize.width;
        
        [self addChild:_badLabel];
        
        _grayField = [SKSpriteNode spriteNodeWithColor:[UIColor colorWithWhite:0.4 alpha:0.7] size:CGSizeMake(1, 1)];
        _grayField.zPosition = 20;
    }
    
    return self;
}

- (void)setCharacter:(Character *)character
{
    _character = character;
    
    [_imageNode removeFromParent];
    
    _imageNode = character.sprite;
    
    [_grayField removeFromParent];
    if (self.character.isBusy) {
        _grayField.position = _imageNode.position;
        _grayField.size = _imageNode.size;
        [self addChild:_grayField];
    }
    
    [_imageNode removeFromParent];
    [self addChild:_imageNode];
    
    [self updateInfo];
}

- (void)setGoldDisplay:(GoldDisplayType)goldDisplay
{
    _goldDisplay = goldDisplay;
    
    [self updateInfo];
}

- (void)updateInfo
{
    _nameLabel.text = self.character.fullName;
    
    switch (self.goldDisplay) {
        case GoldDisplayActionCost:
            //_goldLabel.text = [NSString stringWithFormat:@"Price for good task: %ld\nPrice for bad task: %ld", self.character.goodActionCost, self.character.badActionCost];
            _goodLabel.text = [NSString stringWithFormat:@"Цена доброго дела: %ld", self.character.goodActionCost];
            _badLabel.text = [NSString stringWithFormat:@"Цена злого дела: %ld", self.character.badActionCost];
            break;
            
        case GoldDisplaySalary:
            //_goldLabel.text = [NSString stringWithFormat:@"Salary: %ld", self.character.salary];
            _goodLabel.text = [NSString stringWithFormat:@"Жалованье: %ld", self.character.salary];
            _goodLabel.fontColor = [UIColor whiteColor];
            break;
    }
    
    _nameLabel.position = CGPointMake(0, 0.5 * (CharacterNodeSize.height - _nameLabel.size.height));
    _goodLabel.position = CGPointMake(0, 0.5 * (_goodLabel.size.height - CharacterNodeSize.height) - 20);
    _badLabel.position = CGPointMake(0, 0.5 * (_goodLabel.size.height - CharacterNodeSize.height - _goodLabel.size.height) - 60);
    CGSize imageSize = CGSizeMake(CharacterNodeSize.width, CharacterNodeSize.height - _nameLabel.size.height - _goodLabel.size.height);
    if (imageSize.width / imageSize.height < _imageNode.texture.size.width / _imageNode.texture.size.height) {
        imageSize.height = _imageNode.texture.size.height * imageSize.width / _imageNode.texture.size.width;
    }
    else {
        imageSize.width = _imageNode.texture.size.width * imageSize.height / _imageNode.texture.size.height;
    }
    
    _imageNode.size = imageSize;
    _imageNode.position = CGPointMake(0, 0.5 * (_goodLabel.size.height - _nameLabel.size.height));
}

- (CGSize)size
{
    return CharacterNodeSize;
}

@end
