//
//  CityManager.m
//  LordGame
//
//  Created by Michael Dyachenko on 24/01/15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "CityManager.h"
#import "NameManager.h"

static const NSInteger CITY_AMOUNT = 9;
static const NSInteger ROW_AMOUNT = 3;
static const NSInteger COLUMN_AMOUNT = 3 ;

@implementation CityManager
{
    NSMutableArray* _cityList;
}

+ (instancetype)shared
{
    static CityManager* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        instance = [[CityManager alloc] init];
    });
    
    return instance;
}

- (instancetype) init
{
    if(( self = [super init] ))
    {
        [self reset];
    }
    
    return self;
}

- (NSArray *) cityList
{
    return _cityList;
}

- (Money) incomeForAllCities
{
    Money total = 0;
    for(City *city in _cityList) {
        total += city.income;
    }
    
    return total;
}

- (void)chooseEventCity
{
    _eventCity = _cityList[arc4random() % _cityList.count];
}

- (void) updatePopulation
{
    for(City *city in _cityList)
    {
        [city updatePopulation];
    }
}

- (void)reset
{
    float offset = 120.0f;
    _cityList = [NSMutableArray new];
    NSArray *nameList = [[NameManager sharedManager] cityNameList: CITY_AMOUNT];
    CGSize deviceSize = [UIScreen mainScreen].bounds.size;
    CGSize cellSize = CGSizeMake((deviceSize.width - 2* offset) / COLUMN_AMOUNT,( deviceSize.height-  offset)  / ROW_AMOUNT);
    for(int cityNumber = 0; cityNumber < CITY_AMOUNT; cityNumber++) {
        int row = cityNumber/COLUMN_AMOUNT;
        int column = cityNumber%COLUMN_AMOUNT;
        
        CGPoint location = CGPointMake(offset - 20 + cellSize.width/2 + cellSize.width*column - deviceSize.width/2 + (cellSize.width/4 - arc4random()%((int)(cellSize.width*0.4))), offset - 20 + cellSize.height/2 + cellSize.height*row - deviceSize.height/2 + (cellSize.height/4 - arc4random()%((int)(cellSize.height*0.5))));
        
        NSString *name = nameList[cityNumber];
        City *city = [[City alloc] initWithName:name];
        city.mapLocation = location;
        city.size = CGSizeMake(cellSize.width / 3, cellSize.height / 3);
        [_cityList addObject:city];
    }
}

@end
