//
//  Event.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "Event.h"
#import "Reaction.h"
#import "CharacterMenu.h"
#import "CharacterManager.h"
#import "CityManager.h"

static NSString* const EventTextKey = @"Text";
static NSString* const EventTypeKey = @"Type";
static NSString* const EventEmptyChoiseKey = @"Empty choise";
static NSString* const EventReactionListKey = @"Reaction list";

@implementation Event
{
    Reaction* _emptyReaction;
    NSDictionary* _reactionsData;
}

- (instancetype)initWithData:(NSDictionary *)eventData
{
    self = [super init];
    
    if (self != nil) {
        NSString* cityName = [CityManager shared].eventCity.name;
        
        _text = eventData[EventTextKey];
        _text = [_text stringByReplacingOccurrencesOfString:@"<City>" withString:cityName];
        
        _eventStat.event = [eventData[EventTypeKey] integerValue];
        _emptyReaction = [[Reaction alloc] initWithData:eventData[EventEmptyChoiseKey]];
        _emptyAnswer = _emptyReaction.text;
        _reactionsData = eventData[EventReactionListKey];
    }
    
    return self;
}

- (NSString*)goodActionText
{
    Reaction* reaction = [self reaction:YES];
    NSString* actionText = [NSString stringWithFormat:@"%@ (%ld)", reaction.text, [CharacterMenu sharedMenu].selectedCharacter.goodActionCost];
    return actionText;
}

- (NSString*)badActionText
{
    Reaction* reaction = [self reaction:NO];
    NSString* actionText = [NSString stringWithFormat:@"%@ (%ld)", reaction.text, [CharacterMenu sharedMenu].selectedCharacter.badActionCost];
    return actionText;
}

- (Reaction*)reaction:(BOOL)good
{
    CharacterType character = [CharacterMenu sharedMenu].selectedCharacter.type;
    NSDictionary* characterReactionList = nil;
    switch (character) {
        case CharacterKnight:
            characterReactionList = _reactionsData[KnightKey];
            break;
            
        case CharacterMonk:
            characterReactionList = _reactionsData[MonkKey];
            break;
            
        case CharacterMage:
            characterReactionList = _reactionsData[MageKey];
            break;
            
        case CharacterJester:
            characterReactionList = _reactionsData[JesterKey];
            break;
    }
    
    if (good) {
        return [[Reaction alloc] initWithData:characterReactionList[@"Good"]];
    }
    else {
        return [[Reaction alloc] initWithData:characterReactionList[@"Bad"]];
    }
}

- (void)makeEmptyAction
{
    _eventStat.solution = 0;
    [self calcEventResult:_emptyReaction.result];
}

- (void)makeGoodAction
{
    _eventStat.solution = 1;
    Reaction* goodReaction = [self reaction:YES];
    [self calcEventResult:goodReaction.result];
}

- (void)makeBadAction
{
    _eventStat.solution = -1;
    Reaction* badReaction = [self reaction:NO];
    [self calcEventResult:badReaction.result];
}

- (void)calcEventResult:(Result*)result
{
    _eventStat.result = result.type;
    _resultText = result.text;
    
    [[CharacterManager sharedManager] returnCharacters];
    [[CharacterMenu sharedMenu].selectedCharacter upgradeLevel];
    [CharacterMenu sharedMenu].selectedCharacter.specialization += _eventStat.result;
    [CharacterMenu sharedMenu].selectedCharacter.isBusy = YES;
    
    [result execute];
    
    _goodResultList = result.goodResultList;
    _badResultList = result.badResultList;
}

@end
