//
//  EventManager.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "EventManager.h"
#import "Event.h"
#import "CharacterManager.h"
#import "CityManager.h"

static NSArray* _eventDataList = nil;

@implementation EventManager
{
    Event* _curEvent;
}

+ (instancetype)sharedManager
{
    static EventManager* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        instance = [EventManager new];
    });
    
    return instance;
}

- (void)chooseEvent
{
    _eventComplete = NO;
    
    [[CityManager shared] chooseEventCity];
    
    if (_eventDataList == nil) {
        NSString* fileName = [[NSBundle mainBundle] pathForResource:@"EventListRu" ofType:@"plist"];
        _eventDataList = [NSArray arrayWithContentsOfFile:fileName];
    }
    
    NSUInteger eventNumber = arc4random() % _eventDataList.count;
    _curEvent = [[Event alloc] initWithData:_eventDataList[eventNumber]];    
}

- (NSString *)message
{
    return _curEvent.text;
}

-(NSString *)emptyActionName
{
    return _curEvent.emptyAnswer;
}

- (NSString *)goodActionName
{
    return [_curEvent goodActionText];
}

- (NSString *)badActionName
{
    return [_curEvent badActionText];
}

- (void)makeEmptyAction
{
    _eventComplete = YES;
    
    [_curEvent makeEmptyAction];
    
    [[CharacterManager sharedManager].king changeStatistic:_curEvent.eventStat];
}

- (void)makeGoodActionForCharacter:(Character *)character
{
    _eventComplete = YES;
    
    [_curEvent makeGoodAction];
    
    [[CharacterManager sharedManager].king changeStatistic:_curEvent.eventStat];
    [character changeStatistic:_curEvent.eventStat];
}

- (void)makeBadActionForCharacter:(Character *)character
{
    _eventComplete = YES;
    
    [_curEvent makeBadAction];
    
    [[CharacterManager sharedManager].king changeStatistic:_curEvent.eventStat];
    [character changeStatistic:_curEvent.eventStat];
}

- (NSString *)eventResultText
{
    return _curEvent.resultText;
}

- (NSArray *)goodResultList
{
    return _curEvent.goodResultList;
}

- (NSArray *)badResultList
{
    return _curEvent.badResultList;
}

@end
