//
//  FinanceMenu.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "FinanceMenu.h"
#import "Treasury.h"
#import "ThroneScene.h"
#import "CharacterNode.h"
#import "CharacterManager.h"

@implementation FinanceMenu
{
    SKLabelNode* _goldLabel;
    NSMutableArray* _characterList;
}

+ (instancetype)sharedMenu
{
    static FinanceMenu* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        instance = [FinanceMenu new];
        instance.userInteractionEnabled = YES;
    });
    
    return instance;
}

- (void)initMenuInterface
{
    SKSpriteNode* menuBackground = [SKSpriteNode spriteNodeWithColor:[UIColor whiteColor] size:self.scene.size];
    menuBackground.zPosition = 1;
    
    [self addChild:menuBackground];
    
    _goldLabel = [SKLabelNode new];
    _goldLabel.position = CGPointMake(0, 0.47 * self.scene.size.height);
    _goldLabel.fontColor = [UIColor blackColor];
    _goldLabel.zPosition = 2;
    
    [self addChild:_goldLabel];
}

- (void)updateMenu
{
    if (_goldLabel == nil) {
        [self initMenuInterface];
    }
    
    Treasury* treasury = [Treasury sharedTreasury];
    _goldLabel.text = [NSString stringWithFormat:@"%ld + %ld - %ld", treasury.gold, treasury.income, treasury.salary];
    
    if (_characterList == nil) {
        _characterList = [NSMutableArray new];
    }
    
    NSArray* characters = [CharacterManager sharedManager].characterList;
    for (NSUInteger characterIndex = 0; characterIndex < characters.count || characterIndex < _characterList.count; characterIndex++) {
        if (characterIndex >= characters.count) {
            CharacterNode* characterNode = _characterList[characterIndex];
            [characterNode removeFromParent];
            
            [_characterList removeObjectAtIndex:characterIndex];
            characterIndex--;
            
            continue;
        }
        
        Character* character = characters[characterIndex];
        
        CharacterNode* characterNode = nil;
        if (characterIndex < _characterList.count) {
            characterNode = _characterList[characterIndex];
        }
        else {
            characterNode = [CharacterNode new];
            characterNode.goldDisplay = GoldDisplaySalary;
            characterNode.zPosition = 2;
            [_characterList addObject:characterNode];
            [self addChild:characterNode];
        }
        
        characterNode.character = character;
        characterNode.position = CGPointMake((-0.4 + 0.25 * characterIndex) * self.scene.size.width, self.scene.size.height * 0.1);
    }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    Treasury* treasury = [Treasury sharedTreasury];
    treasury.gold += treasury.income - treasury.salary;
    
    [self removeFromParent];
    
    [[ThroneScene sharedScene] returnToThrone];
}

@end
