//
//  KingSprite.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "KingSprite.h"

@implementation KingSprite
{
    SKTexture* _idleTexture;
    SKTexture* _winkTexture;
    SKTexture* _talkTexture;
    
    SKSpriteNode* _hand;
}

- (instancetype)init
{
    self = [super initWithImageNamed:@"king_idle"];
    
    if (self != nil) {
        _idleTexture = self.texture;
        _winkTexture = [SKTexture textureWithImageNamed:@"king_wink"];
        _talkTexture = [SKTexture textureWithImageNamed:@"king_talk"];
        
        _hand = [SKSpriteNode spriteNodeWithImageNamed:@"kings_limb"];
        _hand.zPosition = -0.5;
        _hand.hidden = YES;
        [self addChild:_hand];
        
        [self wink];
    }
    
    return self;
}

- (void)wink
{
    _hand.hidden = YES;
    
    SKAction* delay1Action = [SKAction waitForDuration:0.008 * (arc4random() % 100) + 0.5];
    SKAction* winkAction = [SKAction setTexture:_winkTexture];
    SKAction* delay2Action = [SKAction waitForDuration:0.3];
    SKAction* idleAction = [SKAction setTexture:_idleTexture];
    SKAction* sequence = [SKAction sequence:@[delay1Action, winkAction, delay2Action, idleAction]];
    
    [self runAction:sequence completion:^{
        [self wink];
    }];
}

- (void)speakWithCompletition:(void (^)())handler
{
    SKAction* soundAction = [SKAction playSoundFileNamed:@"king.wav" waitForCompletion:NO];
    [self.scene runAction:soundAction];
    
    _hand.hidden = NO;
    
    [self removeAllActions];
    
    NSMutableArray* sequenceList = [NSMutableArray new];
    
    for (float time = 0; time < 1.3;) {
        SKAction* talkAction = [SKAction setTexture:_talkTexture];
        SKAction* delay1Action = [SKAction waitForDuration:0.004 * (arc4random() % 100) + 0.05];
        SKAction* idleAction = [SKAction setTexture:_idleTexture];
        SKAction* delay2Action = [SKAction waitForDuration:0.004 * (arc4random() % 100) + 0.05];
        
        [sequenceList addObjectsFromArray:@[talkAction, delay1Action, idleAction, delay2Action]];
        time += delay1Action.duration + delay2Action.duration;
    }
    
    SKAction* sequence = [SKAction sequence:sequenceList];
    
    [self runAction:sequence completion:^{
        if (handler != nil) {
            handler();
        }
        
        [self wink];
    }];
}

@end
