//
//  Map.m
//  LordGame
//
//  Created by Michael Dyachenko on 24/01/15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "Map.h"
#import "CityManager.h"
#import "ThroneScene.h"
#import "EventManager.h"
#import "DSMultilineLabelNode.h"
#import "CityNode.h"
#import "NameManager.h"

static NSString *const PLUS_FILENAME = @"plus";
static NSString *const MINUS_FILENAME = @"minus";
static NSString *const FONT_NAME = @"Pixel Cyr";

typedef NS_ENUM(Byte, MapState) {
    MapStateNone,
    MapStateEvent,
    MapStateCities
};

static NSString *const BACKGROUND_FILENAME = @"map";

@implementation Map
{
    SKSpriteNode *_backgroundMapSprite;
    NSMutableArray *_cityList;
    NSMutableArray *_cityObjectList;
    DSMultilineLabelNode* _preloaderText;
    DSMultilineLabelNode* _eventText;
    MapState _mapState;
    
    NSMutableArray *newsList;
}

- (instancetype) init
{
    if((self = [super init])) {
        _mapState = MapStateNone;
        [self initBackground];
        newsList = [NSMutableArray new];
        self.userInteractionEnabled = YES;
        [self initCities];
    }
    
    return self;
}

- (void) initBackground
{
    _backgroundMapSprite = [SKSpriteNode spriteNodeWithImageNamed:BACKGROUND_FILENAME];
    [self addChild:_backgroundMapSprite];
}

- (void) initCities
{
    _cityList = [CityManager shared].cityList.mutableCopy;
    _cityObjectList = [NSMutableArray new];
    
    for(City *city in _cityList) {
        CityNode *cityNode = [[CityNode alloc] initWithCity:city];
        cityNode.position = city.mapLocation;
        [_backgroundMapSprite addChild:cityNode];
        [_cityObjectList addObject:cityNode];
        cityNode.zPosition = 3;
    }
}

- (void)update
{

}

- (void) showMap: (BOOL) isNeedCities
{
    if (isNeedCities == YES) {
        _mapState = MapStateCities;
    }
    else {
        [self hideCities];
        
        _mapState = MapStateEvent;
        
        CGSize deviceSize = [UIScreen mainScreen].bounds.size;
        
        _preloaderText = [DSMultilineLabelNode labelNodeWithFontNamed:FONT_NAME];
        _preloaderText.text = [NameManager sharedManager].preloaderText;
        _preloaderText.fontSize = 35;
        _preloaderText.position = CGPointMake(0, 0.2 * deviceSize.height);
        _preloaderText.paragraphWidth = deviceSize.width * 0.9;
        _preloaderText.fontColor = [UIColor whiteColor];
        _preloaderText.alpha = 0.0f;
        [_backgroundMapSprite addChild:_preloaderText];

        _eventText = [DSMultilineLabelNode labelNodeWithFontNamed:FONT_NAME];
        _eventText.text = [EventManager sharedManager].eventResultText;
        _eventText.fontSize = 35;
        _eventText.position = CGPointMake(0, 0.5 * (_preloaderText.size.height + _eventText.size.height) + 16);
        _eventText.paragraphWidth = deviceSize.width * 0.9;
        _eventText.fontColor = [UIColor whiteColor];
        _eventText.alpha = 0.0f;
        [_backgroundMapSprite addChild:_eventText];
        
        SKAction *pauseAction = [SKAction waitForDuration:0.5];
        SKAction *fadeInAction = [SKAction fadeInWithDuration:0.7];
        SKAction *groupAction = [SKAction sequence:@[pauseAction, fadeInAction]];
        
        SKAction *pauseAction1 = [SKAction waitForDuration:0.3];
        SKAction *fadeInAction1 = [SKAction fadeInWithDuration:1.2];
        SKAction *groupAction1 = [SKAction sequence:@[pauseAction1, fadeInAction1]];
        
        
        [_preloaderText runAction:groupAction1 completion:^{
            [_eventText runAction:groupAction completion:^{
                [self showNews];
            }];
        }];
    }
}

- (void) showNews
{
    NSArray *badNews = [EventManager sharedManager].badResultList;
    float duration = 0.3;
    float fontSize = 25;
    int lastY = _eventText.position.y - 80;
    float offset = 50;
    for (int i = 0; i < badNews.count; i++)
    {
        
        NSString *news = badNews[i];
        DSMultilineLabelNode* newsLabel = [DSMultilineLabelNode labelNodeWithFontNamed:FONT_NAME];
        newsLabel.text = news;
        newsLabel.fontSize = fontSize;
        newsLabel.paragraphWidth = 400;
        newsLabel.position = CGPointMake(self.scene.size.width * 0.05 + newsLabel.size.width/2, lastY);
        lastY -= newsLabel.size.height/2 + offset;
        newsLabel.horizontalAlignmentMode = SKLabelHorizontalAlignmentModeLeft;
        newsLabel.fontColor = [UIColor redColor];
        [ newsLabel runAction:[SKAction fadeInWithDuration:duration]];
        [self addChild:newsLabel];
        [newsList addObject:newsLabel];
    }
    
    lastY = _eventText.position.y - 80;
    NSArray *goodNews = [EventManager sharedManager].goodResultList;
    for(int i = 0; i < goodNews.count; i++)
    {
        NSString *news = goodNews[i];
        DSMultilineLabelNode* newsLabel = [DSMultilineLabelNode labelNodeWithFontNamed:FONT_NAME];
        newsLabel.text = news;
        newsLabel.paragraphWidth = 400;
        newsLabel.fontSize = fontSize;
        newsLabel.position = CGPointMake(-self.scene.size.width * 0.4 + newsLabel.size.width/2, lastY);
        lastY -= newsLabel.size.height/2 + offset;
        newsLabel.horizontalAlignmentMode = SKLabelHorizontalAlignmentModeLeft;
        newsLabel.fontColor = [UIColor greenColor];
         [ newsLabel runAction:[SKAction fadeInWithDuration:duration]];
        [self addChild:newsLabel];
        [newsList addObject:newsLabel];
    }
}

- (void) hideCities
{
    for(CityNode *cityNode in _cityObjectList)
    {
        cityNode.hidden = YES;
        cityNode.alpha = 0.0;
    }
}

- (void) showCities
{
    for (CityNode *cityNode in _cityObjectList) {
        cityNode.hidden = NO;
        cityNode.alpha = 0.0;
        SKAction *fadeInAction = [SKAction fadeInWithDuration:0.3];
        [cityNode runAction:fadeInAction];
    }
    
    _mapState = MapStateCities;
}

- (void)closeMap
{
    [self removeFromParent];
    [[ThroneScene sharedScene] returnToThrone];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    if(_mapState == MapStateCities)
    {
         [self closeMap];
    }
    else if(_mapState == MapStateEvent)
    {
        SKAction *pauseAction = [SKAction waitForDuration:0.1];
        SKAction *fadeOutAction = [SKAction fadeOutWithDuration:0.5];
        SKAction *remove = [SKAction removeFromParent];
        SKAction *groupAction = [SKAction sequence:@[fadeOutAction, pauseAction, remove]];
        [_eventText runAction:groupAction completion:^(void){[self showCities];}];
        [_preloaderText runAction:[SKAction sequence:@[fadeOutAction, remove]]];
        
        for(DSMultilineLabelNode *newsLabel in newsList)
        {
           
            SKAction *fadeOutAction1 = [SKAction fadeOutWithDuration:0.5];
            SKAction *remove1 = [SKAction removeFromParent];
            SKAction *groupAction1 = [SKAction sequence:@[fadeOutAction1, remove1]];
            [newsLabel runAction:groupAction1];
        }
        
        [newsList removeAllObjects];
    }
}

@end
