//
//  MessengerSprite.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "MessengerSprite.h"
static NSArray *messengerTextureList;
@implementation MessengerSprite
{
    SKTexture* _standTexture;
    SKTexture* _walkTexture;
    SKTexture* _talkTexture;
    SKTexture* _winkTexture;
}

- (instancetype)init
{
    self = [super init];
    
    if (self != nil) {
        self.type = MessengerPeon;
        if(messengerTextureList == nil) {
            messengerTextureList = @[@"man", @"knight", @"priest", @"mage", @"jester"];
        }
    }
    
    return self;
}

- (void)setType:(MessengerType)type
{
    _type = type;
    NSString *filename = messengerTextureList[_type];
    
    if (_type == MessengerPeon) {
        int index = arc4random()%3;
        filename = [NSString stringWithFormat:@"man%02d", index + 1];
    }
    
    _standTexture = [SKTexture textureWithImageNamed:[NSString stringWithFormat:@"%@_stand", filename]];
    _walkTexture = [SKTexture textureWithImageNamed:[NSString stringWithFormat:@"%@_walk", filename]];
    
    if (_type == MessengerPeon) {
        _talkTexture = [SKTexture textureWithImageNamed:[NSString stringWithFormat:@"%@_talk", filename]];
    }
    
    _winkTexture = [SKTexture textureWithImageNamed:[NSString stringWithFormat:@"%@_wink", filename]];
    
    if (CGSizeEqualToSize(self.size, CGSizeZero)) {
        self.size = _standTexture.size;
    }
    
    [self wink];
}

- (void)speakWithCompletition:(void (^)())handler
{
    SKAction* soundAction = [SKAction playSoundFileNamed:@"peasant.wav" waitForCompletion:NO];
    [self.scene runAction:soundAction];
    
    self.walk = NO;
    
    [self removeAllActions];
    
    NSMutableArray* sequenceList = [NSMutableArray new];
    
    for (float time = 0; time < 1.3;) {
        SKAction* talkAction = [SKAction setTexture:_talkTexture];
        SKAction* delay1Action = [SKAction waitForDuration:0.004 * (arc4random() % 100) + 0.05];
        SKAction* idleAction = [SKAction setTexture:_standTexture];
        SKAction* delay2Action = [SKAction waitForDuration:0.004 * (arc4random() % 100) + 0.05];
        
        [sequenceList addObjectsFromArray:@[talkAction, delay1Action, idleAction, delay2Action]];
        time += delay1Action.duration + delay2Action.duration;
    }
    
    SKAction* sequence = [SKAction sequence:sequenceList];
    
    [self runAction:sequence completion:^{
        if (handler != nil) {
            handler();
        }
        
        [self wink];
    }];
}

- (void)setWalk:(BOOL)walk
{
    [self removeAllActions];
    
    _walk = walk;
    
    if (self.walk) {
        SKAction* walkAction = [SKAction animateWithTextures:@[_standTexture, _walkTexture] timePerFrame:0.17];
        SKAction* loopAction = [SKAction repeatActionForever:walkAction];
        
        [self runAction:loopAction];
    }
    else {
        self.texture = _standTexture;
        if(_type != MessengerPeon)
        {
            [self wink];
        }
    }
}

- (void)wink
{
    SKAction* delay1Action = [SKAction waitForDuration:0.008 * (arc4random() % 100) + 0.5];
    SKAction* winkAction = [SKAction setTexture:_winkTexture];
    SKAction* delay2Action = [SKAction waitForDuration:0.3];
    SKAction* idleAction = [SKAction setTexture:_standTexture];
    SKAction* sequence = [SKAction sequence:@[delay1Action, winkAction, delay2Action, idleAction]];
    
    [self runAction:sequence completion:^{
        [self wink];
    }];
}

@end
