//
//  NameManager.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "NameManager.h"

@implementation NameManager

static NSArray* _cityNameList = nil;
static NSDictionary* _characterNameList = nil;
static NSDictionary* _bynameList = nil;

static NSString* const BynameKey = @"Title";
static NSString* const BynameDescriptionKey = @"Description";

+ (instancetype)sharedManager
{
    static NameManager* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        instance = [NameManager new];
    });
    
    return instance;
}

- (NSArray *)cityNameList: (int) cityAmount
{
    if (_cityNameList == nil) {
        NSString* fileName = [[NSBundle mainBundle] pathForResource:@"CityNameListRu" ofType:@"plist"];
        _cityNameList = [NSArray arrayWithContentsOfFile:fileName];
    }
    
    NSArray* randList = [_cityNameList sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
        if (arc4random() % 2 == 0) {
            return NSOrderedAscending;
        }
        else {
            return NSOrderedDescending;
        }
    }];
    
    NSRange subarrayRange = {0, cityAmount};
    return [randList subarrayWithRange:subarrayRange];
}

//- (NSString *)kingName
//{
//    if (_characterNameList == nil) {
//        NSString* filename = [[NSBundle mainBundle] pathForResource:@"CharacterNameList" ofType:@"plist"];
//        _characterNameList = [NSDictionary dictionaryWithContentsOfFile:filename];
//    }
//    
//    NSArray *concreteTypeNameList = _characterNameList[KingKey];
//    NSUInteger number = arc4random() % concreteTypeNameList.count;
//    
//    return concreteTypeNameList[number];
//}

- (NSString *)characterNameForType:(CharacterType)type
{
    if (_characterNameList == nil) {
        NSString* filename = [[NSBundle mainBundle] pathForResource:@"CharacterNameListRu" ofType:@"plist"];
        _characterNameList = [NSDictionary dictionaryWithContentsOfFile:filename];
    }
    
    NSArray *concreteTypeNameList = nil;
    switch (type) {
        case CharacterJester:
            concreteTypeNameList = _characterNameList[JesterKey];
            break;
            
        case CharacterKnight:
            concreteTypeNameList = _characterNameList[KnightKey];
            break;
            
        case CharacterMage:
            concreteTypeNameList = _characterNameList[MageKey];
            break;
            
        case CharacterMonk:
            concreteTypeNameList = _characterNameList[MonkKey];
            break;
    }
    
    NSUInteger number = arc4random() % concreteTypeNameList.count;
    
    return concreteTypeNameList[number];
}

- (NSString *)messangerName
{
   // NSArray* part1NameList = @[@"Tiny", @"Fat", @"Noodle", @"Cockeyed", @"Sloven", @"Silent", @"Crazy", @"Foolish", @"Greedy", @"Crackpot"];
   // NSArray* part2NameList = @[@"Bob", @"John", @"Scot", @"Jack", @"Bill", @"Stan"];
    
    NSArray* part1NameList = @[@"Малыш", @"Толстый", @"Простак", @"Кривоглазый", @"Грязнуля", @"Тихоня", @"Чокнутый", @"Глупый", @"Хапуга", @"Помешанный"];
     NSArray* part2NameList = @[@"Боб", @"Джон", @"Скот", @"Джэк", @"Билл", @"Стэн"];
    
    NSUInteger index1 = arc4random() % part1NameList.count;
    NSUInteger index2 = arc4random() % part2NameList.count;
    
    return [NSString stringWithFormat:@"%@ %@", part1NameList[index1], part2NameList[index2]];
}

- (NSString *)preloaderText
{
    NSArray* textList = @[@"Летописцы написали о том, что...", @"Вот так и случилось, что...", @"Люди до сих пор болтают о том, как...", @"В том месяце опять..."];
    NSUInteger index = arc4random() % textList.count;
    return textList[index];
}

- (Byname *)kingBynameByStatisticKey:(NSString *)key
{
    if (_bynameList == nil) {
        NSString* filename = [[NSBundle mainBundle] pathForResource:@"BynameListRu" ofType:@"plist"];
        _bynameList = [NSDictionary dictionaryWithContentsOfFile:filename];
    }
    
    NSDictionary* kingBynameList = _bynameList[KingKey];
    NSDictionary* kingBynameData = kingBynameList[key];
    
    Byname* result = [Byname new];
    result.title = kingBynameData[BynameKey];
    result.nameDescription = kingBynameData[BynameDescriptionKey];
    
    return result;
}

- (Byname *)bynameByStatisticKey:(NSString *)key andCharacterType:(CharacterType)type
{
    if (_bynameList == nil) {
        NSString* filename = [[NSBundle mainBundle] pathForResource:@"BynameListRu" ofType:@"plist"];
        _bynameList = [NSDictionary dictionaryWithContentsOfFile:filename];
    }
    
    NSDictionary* characterBynameList = nil;
    switch (type) {
        case CharacterJester:
            characterBynameList = _bynameList[JesterKey];
            break;
            
        case CharacterKnight:
            characterBynameList = _bynameList[KnightKey];
            break;
            
        case CharacterMage:
            characterBynameList = _bynameList[MageKey];
            break;
            
        case CharacterMonk:
            characterBynameList = _bynameList[MonkKey];
            break;
    }
    
    NSDictionary* characterBynameData = characterBynameList[key];
    
    Byname* result = [Byname new];
    result.title = characterBynameData[BynameKey];
    result.nameDescription = characterBynameData[BynameDescriptionKey];
    
    return result;
}

@end
