//
//  Result.m
//  LordGame
//
//  Created by MAXIM KORNEEV on 24.01.15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "Result.h"
#import "CityManager.h"
#import "Treasury.h"
#import "CharacterManager.h"
#import "CharacterMenu.h"

@implementation Result
{
    NSString* _moodModificator;
    NSString* _populationModificator;
    NSString* _goldModificator;
    NSNumber* _characterDeath;
    
    float _chance;
}

- (instancetype)initWithData:(NSDictionary *)resultData
{
    self = [super init];
    
    if (self != nil) {
        NSString* cityName = [CityManager shared].eventCity.name;
        
        _text = resultData[@"Description"];
        _text = [_text stringByReplacingOccurrencesOfString:@"<City>" withString:cityName];
        
        _chance = MAX([resultData[@"Chance"] floatValue], 1);
        _moodModificator = resultData[@"City mood"];
        _populationModificator = resultData[@"City population"];
        _goldModificator = resultData[@"Gold"];
        _characterDeath = resultData[@"Death"];
    }
    
    return self;
}

- (void)execute
{
    Money goldChange = [self findModificatorValue:_goldModificator];
    float moodChange = [self findModificatorValue:_moodModificator];
    float populationChange = [self findModificatorValue:_populationModificator];
    
    _goodResultList = [NSArray new];
    _badResultList = [NSArray new];
    
    if (goldChange > 0) {
        //_goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"You gain %ld gold", goldChange]];
        _goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"Вы получили %ld золотых", goldChange]];
    }
    if (goldChange < 0) {
        //_badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"You lost %ld gold", goldChange]];
        _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"Вы потеряли %ld золотых", goldChange]];
    }
    
   if (moodChange > 0) {
        //_goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"People from %@ become happy", [CityManager shared].eventCity.name]];
       _goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"Народ города %@ стал счастливее", [CityManager shared].eventCity.name]];
    }
    if (moodChange < 0) {
       // _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"People from %@ become angry", [CityManager shared].eventCity.name]];
         _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"Народ города %@ стал несчастнее", [CityManager shared].eventCity.name]];
    }
    
    if (populationChange > 0) {
        long population = [CityManager shared].eventCity.population * populationChange;
        if(population >= 1000)
        {
           // _goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"Population of %@ increase by %.2fK", [CityManager shared].eventCity.name, 0.001 * money]];
             _goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"Население города %@ выросло на %.2fK", [CityManager shared].eventCity.name, 0.001 * population]];
        }
        else
        {
           // _goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"Population of %@ increase by %ld", [CityManager shared].eventCity.name,  money]];
            _goodResultList = [_goodResultList arrayByAddingObject:[NSString stringWithFormat:@"Население города %@ выросло by %ld", [CityManager shared].eventCity.name,  population]];
        }
   }
    if (populationChange < 0) {
        long population = [CityManager shared].eventCity.population * populationChange;
        if(fabs(population) >= 1000)
        {
            // _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"Population of %@ decrease by %.2fK", [CityManager shared].eventCity.name, -0.001 * money]];
            _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"Население города %@ уменьшилось на %.2fK", [CityManager shared].eventCity.name, -0.001 * population]];
        }
        else
        {
            //_badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"Population of %@ decrease by %ld", [CityManager shared].eventCity.name, -1 * money]];
            _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"Население города %@ уменьшилось на %ld", [CityManager shared].eventCity.name, -1 * population]];
        }
    }
    
    [Treasury sharedTreasury].gold += goldChange;
    [[CityManager shared].eventCity changeEmotion:moodChange];
    [[CityManager shared].eventCity scalePopulation:populationChange];
    
    if (_characterDeath.boolValue) {
        Character* victim = [CharacterMenu sharedMenu].selectedCharacter;
        
        //_badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"%@ is dead", victim.fullName]];
        _badResultList = [_badResultList arrayByAddingObject:[NSString stringWithFormat:@"%@ мертв", victim.fullName]];
        
        [[CharacterManager sharedManager] killCharacter:victim];
    }
}

- (float)findModificatorValue:(NSString*)modificator
{
    NSArray* rangeList = [modificator componentsSeparatedByString:@";"];
    if (rangeList.count < 2) {
        return [rangeList.firstObject floatValue];
    }
    
    float minValue = [rangeList.firstObject floatValue];
    float maxValue = [rangeList.lastObject floatValue];
    
    float random = (maxValue - minValue) * (float)(arc4random() % UINT32_MAX) / UINT32_MAX;
    return minValue + random;
}

- (float)chance
{
    Character* character = [CharacterMenu sharedMenu].selectedCharacter;
    if ((self.type < 0 && character.specialization < 0) || (self.type > 0 && character.specialization > 0)) {
        return _chance * (1 + 0.5 * ABS(character.specialization));
    }
    
    return _chance;
}

@end
