//
//  ThroneScene.m
//  LordGame
//
//  Created by Michael Dyachenko on 24/01/15.
//  Copyright (c) 2015 AppBit. All rights reserved.
//

#import "ThroneScene.h"
#import "EventManager.h"
#import "CityManager.h"
#import "CharacterManager.h"
#import "Map.h"
#import "CharacterMenu.h"
#import "FinanceMenu.h"
#import "MapButton.h"
#import "Treasury.h"
#import "DSMultilineLabelNode.h"
#import "KingSprite.h"
#import "MessengerSprite.h"
#import "NameManager.h"
#import "BubbleSprite.h"
#import "ZPositionList.h"
#import <AVFoundation/AVFoundation.h>
static NSUInteger const GameLength = 12 * 30;
//static NSString *const FONT_NAME = @"DPix_8pt";

typedef NS_ENUM(Byte, GameState) {
    GameStateNewTitul,
    GameStateStartEvent,
    GameStateChooseCharacter,
    GameStateChooseAction,
    GameStateShowResult,
    GameStateShowFinance,
    GameStateGameOver
};

@implementation ThroneScene
{
    GameState _state;
    
    KingSprite* _kingSprite;
    DSMultilineLabelNode* _kingName;
    NSMutableArray* _chooseList;
    BubbleSprite* _kingBubble;
    MessengerSprite* _messengerSprite;
    DSMultilineLabelNode* _messageNode;
    BubbleSprite* _messageBubble;
    DSMultilineLabelNode* _messageName;
    MessengerSprite* _patrialSprite;
    DSMultilineLabelNode* _patrialName;
    
    Map* _map;
    MapButton* _mapButton;
    DSMultilineLabelNode* _titulMessage;
    DSMultilineLabelNode* _scoreLabel;
    SKSpriteNode* _blackSprite;
    NSUInteger _timeCounter;
    AVAudioPlayer *_player;
}

+ (instancetype)sharedScene
{
    static ThroneScene* instance = nil;
    static dispatch_once_t token = 0;
    dispatch_once(&token, ^{
        CGSize deviceSize = [UIScreen mainScreen].bounds.size;
        instance = [[ThroneScene alloc] initWithSize:deviceSize];
    });
    
    return instance;
}

- (instancetype)initWithSize:(CGSize)size
{
    self = [super initWithSize:size];
    
    if (self != nil) {
        [self initThroneScene];
    }
    
    return self;
}

#pragma mark- Drawing methods

- (void)initThroneScene
{
    [self initThroneBackground];
    [self initKing];
    [self initMessanger];
    [self initPatrial];
    [self initMusic];
    _mapButton = [MapButton new];
    _mapButton.zPosition = ZPositionMapButton;
    _mapButton.position = CGPointMake(self.size.width - 0.75 * _mapButton.size.width, self.size.height - 0.75 * _mapButton.size.height);
    
    _blackSprite = [SKSpriteNode spriteNodeWithColor:[UIColor blackColor] size:self.size];
    [self addChild:_blackSprite];
    _blackSprite.alpha = 0.0f;
    _blackSprite.zPosition = ZPositionBlack1;
    _blackSprite.position = CGPointMake(self.size.width/2, self.size.height/2);
    [self addChild:_mapButton];
    
    [self addChild:[Treasury sharedTreasury].goldNode];
    [Treasury sharedTreasury].goldNode.zPosition = ZPositionMoney;
    [Treasury sharedTreasury].goldNode.position = CGPointMake(0.5 * self.size.width, self.size.height - 30);
}

- (void) initMusic
{
    NSURL *url = [NSURL fileURLWithPath:[[NSBundle mainBundle]
                                         pathForResource:@"music"
                                         ofType:@"mp3"]];
    _player = [[AVAudioPlayer alloc] initWithContentsOfURL:url error:nil];
    _player.numberOfLoops = -1;
    [_player play];
}

- (SKSpriteNode*)fireSprite
{
    SKSpriteNode* fireSprite = [SKSpriteNode spriteNodeWithImageNamed:@"flame01"];
    
    NSArray* textureList = @[[SKTexture textureWithImageNamed:@"flame01"],
                             [SKTexture textureWithImageNamed:@"flame02"],
                             [SKTexture textureWithImageNamed:@"flame03"],
                             [SKTexture textureWithImageNamed:@"flame02"]];
    
    SKAction* animateAction = [SKAction animateWithTextures:textureList timePerFrame:1.0 / 12];
    SKAction* loopAction = [SKAction repeatActionForever:animateAction];
    
    [fireSprite runAction:loopAction];
    
    return fireSprite;
}


- (void)initThroneBackground
{
    SKSpriteNode* background = [SKSpriteNode spriteNodeWithImageNamed:@"ThroneBackground"];
    background.anchorPoint = CGPointZero;
    background.size = self.size;
    [self addChild:background];
    
    SKSpriteNode* flame = [self fireSprite];
    flame.position = CGPointMake(648, 368);
    [self addChild:flame];
}

- (void)initKing
{
    _kingSprite = [KingSprite new];
    _kingSprite.position = CGPointMake(104, 272);
    _kingSprite.zPosition = ZPositionPatrial;
    [self addChild:_kingSprite];
    
    _kingName = [DSMultilineLabelNode new];
    _kingName.fontColor = [UIColor whiteColor];
    _kingName.zPosition = ZPositionText;
    _kingName.position = CGPointMake(_kingSprite.position.x, _kingSprite.position.y - 200);
    [self addChild:_kingName];
}

- (void)initMessanger
{
    _messengerSprite = [MessengerSprite new];
    _messengerSprite.zPosition = ZPositionMessenger;
    
    _messageNode = [DSMultilineLabelNode new];
    _messageNode.position = CGPointMake(0.75 * self.size.width, 0.7 * self.size.height);
    _messageNode.paragraphWidth = 0.4 * self.size.width;
    _messageNode.fontColor = [UIColor whiteColor];
    [self addChild:_messageNode];
    
    _messageName = [DSMultilineLabelNode new];
    _messageName.zPosition = ZPositionText;
    _messageName.fontColor = [UIColor whiteColor];
}

- (void)initPatrial
{
    _patrialSprite = [MessengerSprite new];
    _patrialSprite.zPosition = ZPositionPatrial;
    
    _patrialName = [DSMultilineLabelNode new];
    _patrialName.zPosition = ZPositionText;
    _patrialName.fontColor = [UIColor whiteColor];
}

- (void)clearScene
{
    [_messengerSprite removeFromParent];
    [_patrialSprite removeFromParent];
    [_scoreLabel removeFromParent];
    
    [self hideMessage];
    [self showCustomChoose:nil];
}

#pragma mark- Animation methods

- (void)showNewTitulMessage
{
    [self setInteraction:NO];
    
    if (_titulMessage == nil) {
        _titulMessage = [DSMultilineLabelNode new];
        _titulMessage.fontColor = [UIColor whiteColor];
        _titulMessage.position = CGPointMake(0.5 * self.size.width, 0.5 * self.size.height);
        _titulMessage.alpha = 0.0;
        [self addChild:_titulMessage];
    }
    
    King* king = [CharacterManager sharedManager].king;
    _titulMessage.text = [NSString stringWithFormat:@"Отныне среди простого люда вы стали известны как:\n%@", king.fullName];
    
    SKAction* showAction = [SKAction fadeInWithDuration:0.4];
    [_titulMessage runAction:showAction completion:^{
        [self setInteraction:YES];
    }];
}

- (void)hideNewTitulMessage
{
    [self setInteraction:NO];
    
    SKAction* hideAction = [SKAction fadeOutWithDuration:0.4];
    [_titulMessage runAction:hideAction completion:^{
        [self setInteraction:YES];
    }];
}

- (void)showDateWithCompletition:(void(^)())handler
{
    DSMultilineLabelNode* dateLabel = [DSMultilineLabelNode new];
    
    //NSArray* monthList = @[@"Jan", @"Feb", @"Mar", @"Apr", @"May", @"Jun", @"Jul", @"Aug", @"Sep", @"Okt", @"Nov", @"Dec"];
    NSArray* monthList = @[@"Январь", @"Февраль", @"Март", @"Апрель", @"Май", @"Июнь", @"Июль", @"Август", @"Сентябрь", @"Октябрь", @"Ноябрь", @"Декабрь"];
    dateLabel.text = [NSString stringWithFormat:@"%@, %u", monthList[_timeCounter % 12], 1500 + _timeCounter / 12];
    dateLabel.position = CGPointMake(0.5 * self.size.width, 0.5 * self.size.height);
    dateLabel.zPosition = ZPositionText;
    dateLabel.alpha = 0.0;
    dateLabel.fontSize = 35;
    [self addChild:dateLabel];
    
    SKAction* fadeIn = [SKAction fadeInWithDuration:0.8];
    SKAction* delay = [SKAction waitForDuration:1.5];
    SKAction* fadeOut = [SKAction fadeOutWithDuration:0.8];
    SKAction* dateAnimation = [SKAction sequence:@[fadeIn, delay, fadeOut]];
    
    [dateLabel runAction:dateAnimation completion:handler];
}

- (void)showMessengerWithCompletition:(void(^)())handler;
{
    if (_messengerSprite.parent == nil) {
        [self addChild:_messengerSprite];
    }
    if (_messageName.parent == nil) {
        [self addChild:_messageName];
    }
    
    _messengerSprite.type = MessengerPeon;
    _messengerSprite.position = CGPointMake(self.size.width + 40, 280);
    _messengerSprite.xScale = 1.0;
    _messengerSprite.walk = YES;
    
    _messageName.alpha = 1.0;
    _messageName.position = CGPointMake(self.size.width + 40, 72);
    _messageName.text = [NameManager sharedManager].messangerName;
    
    SKAction* moveAction = [SKAction moveByX:-552 y:0 duration:2.6];
    [_messageName runAction:moveAction];
    [_messengerSprite runAction:moveAction completion:^{
        _messengerSprite.walk = NO;
        
        [self showMessage];
        [_messengerSprite speakWithCompletition:^{
            handler();
        }];
    }];
}

- (void)showPatrial:(Character*)patrial withCompletition:(void(^)())handler;
{
    switch (patrial.type) {
        case CharacterJester:
            _patrialSprite.type = MessengerJester;
            break;
            
        case CharacterKnight:
            _patrialSprite.type = MessengerKnight;
            break;
            
        case CharacterMage:
            _patrialSprite.type = MessengerMage;
            break;
            
        case CharacterMonk:
            _patrialSprite.type = MessengerMonk;
            break;
    }
    
    if (_patrialSprite.parent == nil) {
        [self addChild:_patrialSprite];
    }
    if (_patrialName.parent == nil) {
        [self addChild:_patrialName];
    }
    
    _patrialSprite.position = CGPointMake(self.size.width + 40, 256);
    _patrialSprite.xScale = 1;
    _patrialSprite.walk = YES;
    
    _patrialName.position = CGPointMake(self.size.width + 40, 72);
    _patrialName.text = patrial.fullName;
    
    SKAction* moveAction = [SKAction moveByX:-496 y:0 duration:2.4];
    [_patrialName runAction:moveAction];
    [_patrialSprite runAction:moveAction completion:^{
        _patrialSprite.walk = NO;
        
        handler();
    }];
    
    SKAction* waitAction = [SKAction waitForDuration:2.0];
    SKAction* hideAction = [SKAction fadeOutWithDuration:0.3];
    SKAction* sequence = [SKAction sequence:@[waitAction, hideAction]];
    [_messageName runAction:sequence];
}

- (void)hideAllPeoplesWithCompletition:(void(^)())handler;
{
    [_kingSprite speakWithCompletition:^{
        [self showCustomChoose:nil];
        [self hideMessage];
        
        _messengerSprite.xScale = -1;
        _messengerSprite.walk = YES;
        
        SKAction* moveAction = [SKAction moveByX:592 y:0 duration:2.6];
        
        [_messengerSprite runAction:moveAction completion:handler];
        [_messageName runAction:moveAction];
        
        if (_patrialSprite.parent == self) {
            SKAction* delayAction = [SKAction waitForDuration:0.5];
            SKAction* walkAction = [SKAction runBlock:^(void){
                _patrialSprite.walk = YES;
                _patrialSprite.xScale = -1;
                [_patrialSprite runAction:moveAction];
                [_patrialName runAction:moveAction];
            }];
            
            SKAction* sequenceAction = [SKAction sequence:@[delayAction, walkAction]];
            [_patrialSprite runAction:sequenceAction];
        }
    }];
}

- (void)hidePatrialWithCompletition:(void(^)())handler;
{
    [_kingSprite speakWithCompletition:^{
        [self showCustomChoose:nil];
        [self hideMessage];
        
        if (_patrialSprite.parent == self) {
            _patrialSprite.walk = YES;
            _patrialSprite.xScale = -1;
            
            SKAction* moveAction = [SKAction moveByX:276 y:0 duration:1.3];
            [_patrialName runAction:moveAction];
            [_patrialSprite runAction:moveAction completion:handler];
            
            SKAction* waitAction = [SKAction waitForDuration:0.2];
            SKAction* showAction = [SKAction fadeInWithDuration:0.3];
            SKAction* sequence = [SKAction sequence:@[waitAction, showAction]];
            [_messageName runAction:sequence];
        }
    }];
}

#pragma mark- Action methods

- (void)didMoveToView:(SKView *)view
{
    [self startEvent];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (_state == GameStateGameOver) {
        [self resetGame];
        return;
    }
    
    if (_state == GameStateNewTitul) {
        [self hideNewTitulMessage];
        [self startNewMonth];
        return;
    }
    
    UITouch* touch = [touches anyObject];
    CGPoint touchPoint = [touch locationInNode:self];
    
    for (NSUInteger chooseIndex = 0; chooseIndex < _chooseList.count; chooseIndex++) {
        DSMultilineLabelNode* chooseLabel = _chooseList[chooseIndex];
        CGRect chooseFrame = chooseLabel.frame;
        if (CGRectContainsPoint(chooseFrame, touchPoint)) {
            [self makeAction:chooseIndex];
            return;
        }
    }
}

- (void)makeAction:(NSUInteger)chooseIndex
{
    DSMultilineLabelNode* chooseNode = _chooseList[chooseIndex];
    chooseNode.fontColor = [UIColor colorWithRed:75.0 / 255.0 green:150 / 255.0 blue:150 / 255.0 alpha:1.0];
    for(DSMultilineLabelNode *node in _chooseList)
    {
        if(chooseNode != node)
        {
            node.alpha = 0.25;
           // [node runAction:[SKAction fadeOutWithDuration:0.2]];
        }
    }
    switch (_state) {
        case GameStateStartEvent: {
            if (chooseIndex == 0) {
                [self moveToCharacterMenu];
            }
            else {
                [self setInteraction:NO];
                
                [self hideAllPeoplesWithCompletition:^{
                    [self setInteraction:YES];
                    
                    _state = GameStateShowResult;
                    [[EventManager sharedManager] makeEmptyAction];
                    [self doVolumeFade];
                    [self showMap:NO];
                }];
            }
            break;
        }
            
        case GameStateChooseAction: {
            Character* patrial = [CharacterMenu sharedMenu].selectedCharacter;
            
            if (chooseIndex == 0) {
                [self setInteraction:NO];
                
                [self hideAllPeoplesWithCompletition:^{
                    [self setInteraction:YES];
                    
                    if ([Treasury sharedTreasury].gold >= patrial.goodActionCost) {
                        _state = GameStateShowResult;
                        
                        [Treasury sharedTreasury].gold -= patrial.goodActionCost;
                        [[EventManager sharedManager] makeGoodActionForCharacter:patrial];
                        [self doVolumeFade];
                        [self showMap:NO];
                    }
                }];
            }
            else if (chooseIndex == 1) {
                [self setInteraction:NO];
                
                [self hideAllPeoplesWithCompletition:^{
                    [self setInteraction:YES];
                    
                    if ([Treasury sharedTreasury].gold >= patrial.badActionCost) {
                        _state = GameStateShowResult;
                        
                        [Treasury sharedTreasury].gold -= patrial.badActionCost;
                        [[EventManager sharedManager] makeBadActionForCharacter:patrial];
                        [self doVolumeFade];
                        [self showMap:NO];
                    }
                }];
            }
            else {
                [self setInteraction:NO];
                
                [self hidePatrialWithCompletition:^{
                    [self setInteraction:YES];
                    
                    [self moveToCharacterMenu];
                }];
            }
            
            break;
        }
            
        default:
            break;
    }
}

- (void)resetGame
{
    _timeCounter = 0;
    
    [[CharacterManager sharedManager] reset];
    [[CityManager shared] reset];
    
    [self startEvent];
}

- (void)setInteraction:(BOOL)enable
{
    self.userInteractionEnabled = enable;
    _mapButton.userInteractionEnabled = enable;
    
    SKAction* alphaAction = [SKAction fadeAlphaTo:(enable ? 1.0 : 0.0) duration:0.4];
    [_mapButton runAction:alphaAction];
}

#pragma mark- Menu methods

- (void)startEvent
{
    [self clearScene];
    
    _kingName.text = [CharacterManager sharedManager].king.fullName;
    if ([CharacterManager sharedManager].king.newName) {
        _state = GameStateNewTitul;
        [self showNewTitulMessage];
    }
    else {
        _state = GameStateStartEvent;
        [self startNewMonth];
    }
}

- (void)startNewMonth
{
    [self setInteraction:NO];
    
    _state = GameStateStartEvent;
    
    [[CityManager shared] updatePopulation];
    [[EventManager sharedManager] chooseEvent];
    
    [self showDateWithCompletition:^{
        [self showMessengerWithCompletition:^{
            [self setInteraction:YES];
            [self showChoose];
        }];
    }];
}

- (void)showMessage
{
    _messageNode.text = [EventManager sharedManager].message;
    _messageNode.alpha = 0.0;
    
    SKAction* showAction = [SKAction fadeInWithDuration:0.5];
    [_messageNode runAction:showAction];
    
    [_messageBubble removeFromParent];
    _messageBubble = [[BubbleSprite alloc] initWithContent:_messageNode];
    [self addChild:_messageBubble];
}

- (void)hideMessage
{
    _messageNode.text = nil;
    
    [_messageBubble removeFromParent];
}

- (void)showChoose
{
    switch (_state) {
        case GameStateStartEvent: {
            NSString* emptyActionName = [EventManager sharedManager].emptyActionName;
            //[self showCustomChoose:@[@"I need to think...", emptyActionName]];
            [self showCustomChoose:@[@"Дай-ка подумать...", emptyActionName]];
            
            break;
        }
            
        case GameStateChooseAction: {
            NSString* goodActionName = [[EventManager sharedManager] goodActionName];
            NSString* badActionName = [[EventManager sharedManager] badActionName];
            [self showCustomChoose:@[goodActionName, badActionName, @"Пшел прочь"]];
            
            break;
        }
            
        default:
            break;
    }
}

- (void)showCustomChoose:(NSArray*)textList
{
    for (SKNode* chooseNode in _chooseList) {
        [chooseNode removeFromParent];
    }
    
    if (_chooseList) {
        [_chooseList removeAllObjects];
    }
    else {
        _chooseList = [NSMutableArray new];
    }
    
    CGFloat height = 20 * (textList.count - 1);
    
    for (NSString* chooseText in textList) {
        DSMultilineLabelNode* chooseLabel = [DSMultilineLabelNode new];
        chooseLabel.text = chooseText;
        chooseLabel.horizontalAlignmentMode = SKLabelHorizontalAlignmentModeLeft;
        chooseLabel.paragraphWidth = 0.4 * self.size.width;
        chooseLabel.fontColor = [UIColor colorWithRed:98.0 / 255.0 green:182 / 255.0 blue:191 / 255.0 alpha:1.0];
        [self addChild:chooseLabel];
        
        height += chooseLabel.size.height;
        [_chooseList addObject:chooseLabel];
    }
    
    height = 0.7 * self.size.height + 0.5 * height;
    
    for (DSMultilineLabelNode* chooseLabel in _chooseList) {
        chooseLabel.position = CGPointMake(0.25 * self.size.width - 0.5 * (chooseLabel.paragraphWidth - chooseLabel.size.width), height - 0.5 * chooseLabel.size.height);
        height -= chooseLabel.size.height + 20;
    }
    
    [_kingBubble removeFromParent];
    
    if (_chooseList.count > 0) {
        _kingBubble = [[BubbleSprite alloc] initWithContentList:_chooseList];
        [self addChild:_kingBubble];
    }
}

- (void)moveToCharacterMenu
{
    _state = GameStateChooseCharacter;
    
    [CharacterMenu sharedMenu].position = CGPointMake(0.5 * self.size.width, 0.5 * self.size.height);
    [CharacterMenu sharedMenu].zPosition = ZPositionMenu;
    [self addChild:[CharacterMenu sharedMenu]];
    
    [[CharacterMenu sharedMenu] updateMenu];
}

- (void)showMap:(BOOL)showCities
{
    if (_map.parent == nil) {
        
        if (_map == nil) {
            _map = [Map new];
            _map.position = CGPointMake(0.5 * self.size.width, 0.5 * self.size.height);
            _map.zPosition = ZPositionMap;
        }
        
        [_map update];
        [_map showMap:showCities];
        [self addChild:_map];
        _map.alpha = 0.0f;
        _blackSprite.zPosition = ZPositionBlack2;
        [_blackSprite runAction:[SKAction fadeInWithDuration:0.3] completion:^()
            {
                _map.isBlock = YES;
                [_map runAction:[SKAction fadeInWithDuration:1.0] completion:^(){
                                                                                    _map.isBlock = NO;
                                                                                    _blackSprite.zPosition = ZPositionBlack1;
                                                                                    _blackSprite.alpha = 0.0;}]
                ;}];
        
    }
}

- (void)showFinanceMenu
{
    _state = GameStateShowFinance;
    
    [FinanceMenu sharedMenu].position = CGPointMake(0.5 * self.size.width, 0.5 * self.size.height);
    [FinanceMenu sharedMenu].zPosition = ZPositionMenu;
    [self addChild:[FinanceMenu sharedMenu]];
    
    [[FinanceMenu sharedMenu] updateMenu];
}

- (void)showScore
{
    [self clearScene];
    
    if (_scoreLabel == nil) {
        _scoreLabel = [DSMultilineLabelNode new];
        _scoreLabel.fontColor = [UIColor whiteColor];
    }
    
    King* king = [CharacterManager sharedManager].king;
    _scoreLabel.text = [NSString stringWithFormat:@"You died. Your score: %lu", (unsigned long)king.score];
    [self addChild:_scoreLabel];
}

- (void)returnToThrone
{
    //[self hideMessage];
    [self showCustomChoose:nil];
    [_player play];
    switch (_state) {
        case GameStateStartEvent:
        case GameStateChooseAction:
            [self showChoose];
            break;
            
        case GameStateChooseCharacter: {
            Character* patrial = [CharacterMenu sharedMenu].selectedCharacter;
            if (patrial) {
                _state = GameStateChooseAction;
        
                [self setInteraction:NO];
                
                [self showPatrial:patrial withCompletition:^{
                    [self setInteraction:YES];
                    [self showChoose];
                }];
            }
            else {
                _state = GameStateStartEvent;
                
                [_patrialSprite removeFromParent];
                _messageName.alpha = 1.0;
                
                [self showMessage];
                [self showChoose];
            }
            
            break;
        }
            
        case GameStateShowResult:
            _timeCounter++;
            
            if ([CityManager shared].cityList.count == 0 || _timeCounter == GameLength) {
                _state = GameStateGameOver;
                
                [self showScore];
            }
            else if (_timeCounter % 12 == 0) {
                //[self showFinanceMenu];
            }
            else {
                [self startEvent];
            }
            
            break;
            
        case GameStateShowFinance:
            [self startEvent];
            break;
            
        default:
            break;
            
    }
}

-(void)doVolumeFade
{
    if (_player.volume > 0.1) {
        _player.volume = _player.volume - 0.1;
        [self performSelector:@selector(doVolumeFade) withObject:nil afterDelay:0.02];
    } else {
        // Stop and get the sound ready for playing again
        [_player stop];
        _player.currentTime = 0;
        [_player prepareToPlay];
        _player.volume = 1.0;
    }
}

@end
